use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006002
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
benchmarking
BrowserUk
cachestop
codebase
conf
cpan
CPAN
cryptographic
datastructure
deduper
deduplicated
deduplication
deduplicate
dir
dupfind
filename
filenames
Github
hardlink
hardlinks
html
iface
inodes
listref
listrefs
maxdepth
metadata
multithreaded
namespace
OO
parsable
perlmonks
POSIX
ramcache
STDERR
subclasses
subdirectories
subdirectory
unicode
UTF
weedout
wpass
wpsize
xxhash
Tommy
Butler
devnull
Tom
Metro
tmetro
Tim
King
timk
lib
App
Guts
Threaded
ThreadManagement
Algorithms
MapReduce
Common
Digest
Weed
Overrides
